@@ -2,10 +2,12 @@
 """This is the kospex command line tool."""
 import os
 import os.path
+import json
 import logging
 from datetime import datetime, timezone, timedelta
 from shutil import which
 import click
+import requests
 from kospex_core import Kospex, GitRepo
 import kospex_utils as KospexUtils
 from kospex_git import KospexGit
@@ -14,14 +16,55 @@ import kospex_schema as KospexSchema
 from kospex_dependencies import KospexDependencies
 import krunner_utils as KrunnerUtils
 
-KospexUtils.init()
+# Initialize Kospex environment with enhanced logging
+KospexUtils.init(create_directories=True, setup_logging=True, verbose=False)
 kospex = Kospex()
-log = logging.getLogger(__name__)
-#logging.basicConfig(filename='example.log', encoding='utf-8', level=logging.DEBUG)
-VERSION = "0.0.15" # This value should align with the pyproject.toml version for pip
 
-@click.group()
-def cli():
+# Get logger using the new centralized logging system
+log = KospexUtils.get_kospex_logger('kospex')
+
+
+def _configure_runtime_logging(debug=False, verbose=False, quiet=False, log_console=False):
+    """Configure logging based on runtime CLI flags."""
+    import os
+    import logging
+
+    # Set environment variables for the logging system
+    if debug:
+        os.environ['KOSPEX_LOG_LEVEL'] = 'DEBUG'
+        log.info("Debug logging enabled via CLI flag")
+    elif verbose:
+        os.environ['KOSPEX_LOG_LEVEL'] = 'INFO'
+        log.info("Verbose logging enabled via CLI flag")
+    elif quiet:
+        os.environ['KOSPEX_LOG_LEVEL'] = 'ERROR'
+        log.info("Quiet mode enabled via CLI flag")
+
+    if log_console:
+        os.environ['KOSPEX_CONSOLE_LOGGING'] = 'true'
+        log.info("Console logging enabled via CLI flag")
+
+    # Get a fresh logger instance with the new configuration
+    # Note: This is a simplified approach. For full runtime reconfiguration,
+    # we would need to reset the logging handlers, but this provides
+    # immediate feedback for the current session
+    if debug:
+        log.setLevel(logging.DEBUG)
+    elif verbose:
+        log.setLevel(logging.INFO)
+    elif quiet:
+        log.setLevel(logging.ERROR)
+#logging.basicConfig(filename='example.log', encoding='utf-8', level=logging.DEBUG)
+#VERSION = "0.0.16" # This value should align with the pyproject.toml version for pip
+
+@click.group(invoke_without_command=True)
+@click.version_option(version=Kospex.VERSION)
+@click.option('--debug', is_flag=True, default=False, help="Enable debug logging (overrides config).")
+@click.option('--verbose', '-v', is_flag=True, default=False, help="Enable verbose logging.")
+@click.option('--quiet', '-q', is_flag=True, default=False, help="Suppress most output (errors only).")
+@click.option('--log-console', is_flag=True, default=False, help="Enable console logging in addition to file logging.")
+@click.pass_context
+def cli(ctx, debug, verbose, quiet, log_console):
     """Kospex is a tool for assessing code and git repositories.
 
     It is designed to help understand the structure of code, who are developers and
@@ -31,41 +74,157 @@ def cli():
 
     See also https://kospex.io/
 
+    Global Options:
+      --debug: Enable debug-level logging for detailed troubleshooting
+      --verbose: Enable verbose output and INFO-level logging
+      --quiet: Reduce output to errors only
+      --log-console: Show log messages on console as well as in files
+
     """
+    # Store logging preferences in context for subcommands
+    ctx.ensure_object(dict)
+    ctx.obj['debug'] = debug
+    ctx.obj['verbose'] = verbose
+    ctx.obj['quiet'] = quiet
+    ctx.obj['log_console'] = log_console
+
+    # Apply runtime logging configuration
+    if debug or verbose or quiet or log_console:
+        _configure_runtime_logging(debug, verbose, quiet, log_console)
+
+    if ctx.invoked_subcommand is None:
+        # Default behavior when no command is provided
+        click.echo(ctx.get_help())
+        ctx.exit(0)
 
 @cli.command("init")
-@click.option('-create', is_flag=True, default=False, help="Create the default ~/code or KOSPEX_CODE directory.")
-def kospex_init(create):
+@click.option('--create', is_flag=True, default=False, help="Create the default ~/code or KOSPEX_CODE directory.")
+@click.option('--verbose', is_flag=True, default=False, help="Show detailed initialization status.")
+@click.option('--validate', is_flag=True, default=False, help="Validate Kospex setup and configuration.")
+def kospex_init(create, verbose, validate):
     """
-    Perform basic initial setup.
+    Perform comprehensive Kospex environment initialization.
 
-    Use -create to create the default ~/code or KOSPEX_CODE directory.
+    This command sets up the Kospex directory structure, validates configuration,
+    and initializes the centralized logging system.
 
+    Use --create to create missing directories including ~/code or KOSPEX_CODE.
+    Use --verbose for detailed status information during setup.
+    Use --validate to check the current setup without making changes.
     """
-    kospex_code = KospexUtils.get_kospex_code_path()
-    kconf = KospexUtils.get_kospex_config()
+    log.info("Starting Kospex initialization")
+
+    if validate:
+        # Run validation without making changes
+        validation = KospexUtils.validate_kospex_setup()
+
+        print("\n=== Kospex Setup Validation ===")
+        print(f"Overall Status: {validation['overall_status'].upper()}")
+
+        print("\nEnvironment Variables:")
+        for var, info in validation['environment_vars'].items():
+            status = "✓" if info['set'] else "✗"
+            print(f"  {status} {var}: {info['value'] or 'Not set'}")
+
+        print("\nDirectories:")
+        for name, info in validation['directories'].items():
+            exists = "✓" if info['exists'] else "✗"
+            writable = ""
+            if name == 'kospex_home' and info['exists']:
+                writable = " (writable)" if info['writable'] else " (not writable)"
+            elif name == 'kospex_code' and info['exists']:
+                writable = " (readable)" if info['readable'] else " (not readable)"
+            print(f"  {exists} {info['path']}{writable}")
+
+        if validation['logging']:
+            print("\nLogging System:")
+            if validation['logging'].get('directories_exist'):
+                print("  ✓ Logging directories configured")
+            else:
+                print("  ✗ Logging directories need setup")
+
+        if validation['recommendations']:
+            print("\nRecommendations:")
+            for rec in validation['recommendations']:
+                print(f"  • {rec}")
+
+        return
+
+    # Perform initialization
+    init_result = KospexUtils.init(
+        create_directories=create,
+        setup_logging=True,
+        verbose=verbose
+    )
+
+    if verbose:
+        print("\n=== Initialization Results ===")
+        print(f"KOSPEX_HOME: {init_result['kospex_home']}")
+
+        if init_result['directories_created']:
+            print("\nDirectories Created:")
+            for dir_path in init_result['directories_created']:
+                print(f"  • {dir_path}")
+
+        if init_result['environment_vars_set']:
+            print("\nEnvironment Variables Set:")
+            for var in init_result['environment_vars_set']:
+                print(f"  • {var}")
+
+        if init_result['warnings']:
+            print("\nWarnings:")
+            for warning in init_result['warnings']:
+                print(f"  ⚠ {warning}")
+
+        if init_result['errors']:
+            print("\nErrors:")
+            for error in init_result['errors']:
+                print(f"  ✗ {error}")
+
+    # Check for scc installation
     installed = which('scc')
     if not installed:
-        print("Please install scc from https://github.com/boyter/scc")
-        print("This is used to count lines of code, complexity and determine the language.")
+        log.warning("scc binary not found")
+        print("\n⚠ Please install scc from https://github.com/boyter/scc")
+        print("  This is used to count lines of code, complexity and determine the language.")
+    elif verbose:
+        log.info("scc binary found!")
+        print("\n✓ scc binary found and ready")
+
+    # Handle code directory creation/validation
+    kospex_code = KospexUtils.get_kospex_code_path()
 
     if create:
         if not os.path.exists(kospex_code):
-            os.makedirs(kospex_code)
+            try:
+                os.makedirs(kospex_code, exist_ok=True)
+                log.info(f"Created code directory: {kospex_code}")
+                if verbose:
+                    print(f"✓ Created code directory: {kospex_code}")
+            except OSError as e:
+                log.error(f"Failed to create code directory: {e}")
+                print(f"✗ Failed to create code directory: {e}")
         else:
-            print("Directory already exists.")
+            if verbose:
+                print(f"✓ Code directory already exists: {kospex_code}")
     else:
         if os.path.exists(kospex_code):
-            print("~/code directory exists.")
-            print("If you want to overide the code directory")
-            print(f"edit the {kconf} file or")
-            print("set the KOSPEX_CODE environment variable.\n")
+            if verbose:
+                print(f"✓ Code directory exists: {kospex_code}")
+                print("  To override, edit config file or set KOSPEX_CODE environment variable")
         else:
-            print("Directory does not exist.")
-            print("Run with -create to create either the")
-            print("~/code (default) directory, or")
-            print("a directory specified in the KOSPEX_CODE variable.")
-            print(f"or edit the {kconf} file and change the KOSPEX_CODE evironment variable.\n")
+            log.warning(f"Code directory does not exist: {kospex_code}")
+            print(f"\n⚠ Code directory does not exist: {kospex_code}")
+            print("  Run 'kospex init --create' to create it, or")
+            print("  Set the KOSPEX_CODE environment variable to a different location")
+
+    if not init_result['errors']:
+        log.info("Kospex initialization completed successfully")
+        if not verbose:
+            print("✓ Kospex initialization complete!")
+            print("  Run 'kospex init --validate' to check your setup")
+    else:
+        log.error("Kospex initialization completed with errors")
 
 
 @cli.command("summary")
@@ -79,15 +238,25 @@ def kospex_init(create):
 @click.option('-group', type=click.STRING, help="Name of group (of repos) to query with.")
 @click.option('-org', type=click.STRING, help="Name of Org/Team to query with.")
 @click.option('-debug', is_flag=True, default=False, help="Debug mode.")
+@click.option('--verbose', is_flag=True, default=False, help="Verbose output.")
+@click.pass_context
 #@click.option('-', type=click.STRING, help="Name of group (of repos) to query with.")
 # pylint: disable=unused-argument
-def summary(out,server,email, active, docker, dependencies, email_contains,group,org,debug):
+def summary(ctx, out,server,email, active, docker, dependencies, email_contains,group,org,debug,verbose):
     """ Provide a summary of all the known repositories."""
+    # Respect global logging settings from context
+    global_verbose = ctx.obj.get('verbose', False) if ctx.obj else False
+    is_verbose = verbose or global_verbose
+
+    log.info("Starting repository summary")
+
     params = locals()
     if group and org:
+        log.error("Cannot specify both group and org parameters")
         print("Please specify either a group or an org, not both.")
         exit(1)
     elif group and email:
+        log.error("Cannot specify both group and email parameters")
         print("Please specify either a group or an email, not both.")
         exit(1)
 
@@ -147,25 +316,24 @@ def summary(out,server,email, active, docker, dependencies, email_contains,group
         if unknown > 0:
             print(f"WARNING: {unknown} repos found without an entry in 'repos' table.\n")
 
-
     else:
         print("No repositories found in the kospex DB.\n")
 
-@cli.command("sync")
-@click.option('--no-scc', is_flag=True, default=False, help="Skip scc analysis.")
-@click.argument('repo', type=GitRepo())
-def sync(repo,no_scc):
-    """
-    Sync a single repo to the kospex DB, using the native git commands.
-    """
+# @cli.command("sync")
+# @click.option('--no-scc', is_flag=True, default=False, help="Skip scc analysis.")
+# @click.argument('repo', type=GitRepo())
+# def sync(repo,no_scc):
+#     """
+#     Sync a single repo to the kospex DB, using the native git commands.
+#     """
 
-    installed = which('scc')
-    if not installed:
-        print("Please install scc from https://github.com/boyter/scc")
-        print("This is used to count lines of code, complexity and determine the language.")
-        print("or run sync with --no-scc to skip this step.")
+#     installed = which('scc')
+#     if not installed:
+#         print("Please install scc from https://github.com/boyter/scc")
+#         print("This is used to count lines of code, complexity and determine the language.")
+#         print("or run sync with --no-scc to skip this step.")
 
-    kospex.sync_repo(repo, no_scc=no_scc)
+#     kospex.sync_repo(repo, no_scc=no_scc)
 
 #@cli.command("sync")
 #@click.option('-previous', type=int, help='# Commits to sync from the oldest in kospex DB.')
@@ -485,23 +653,51 @@ def deps(repo, file, directory, out, dev):
 @cli.command("sca")
 @click.option('-repo', type=GitRepo(), help="NOT IMPLEMENTED - File path to git repo.")
 @click.option('-dev', is_flag=True, default=True, help="Include dev/test dependencies. EXPERIMENTAL.")
-@click.option('-save', is_flag=True, default=True, help="Save results to kospex DB.")
+@click.option('-save', is_flag=False, default=True, help="Save results to kospex DB.")
+@click.option('-malware', is_flag=True, default=False, help="Check for malware in dependencies.")
 @click.option('-out', type=click.STRING, help="filename to write CSV results to.")
 @click.argument('file_path', required=False, type=click.Path(exists=True))
-def sca(repo, dev, save, out, file_path):
+def sca(repo, dev, save, malware, out, file_path):
     """
-    Run software composition analysis (SCA) tasks
+    Run a lightweight software composition analysis (SCA) task
+    optionally run the maliciouspackages.com analysis of the dependencies
+    -malware to enable malware analysis using the maliciouspackages.com API
+    Your API_MAT needs to be set in the environment variable API_MAT
     """
-    print("\nEXPERIMENTAL FEATURE: SCA\n")
     params = locals()
-    params['print_table'] = True
+    api_mat = os.environ.get('API_MAT')
 
     if repo:
         print("NOT implemented")
         exit(1)
 
     if file_path:
-         results = kospex.dependencies.assess(file_path,**params)
+        # Handle a dependency file (e.g. package.json, requirements.txt) and check dependencies
+        results = kospex.dependencies.assess(file_path,**params)
+
+        import pprint
+        pprint.PrettyPrinter(indent=4).pprint(results)
+
+        if malware:
+            print("Checking packages for malware on maliciouspackages.com")
+            if not api_mat:
+                 print("API_MAT environment variable not set. Please set it to your maliciouspackages.com API key.")
+                 exit(1)
+
+            for result in results:
+                package_type = result.get('package_type')
+                package_name = result.get('package_name')
+
+                if package_type and package_name:
+                    print(f"Checking {package_type} package: {package_name}")
+                    is_malware = kospex.dependencies.check_malware(package_type, package_name,api_mat)
+                    result['malware'] = is_malware
+                else:
+                    print(f"Skipping package with missing type '{package_type}' or name '{package_name}'")
+
+        kospex.dependencies.print_dependencies_table(results,malware=True)
+
+
     else:
         print("Either -repo REPO or file_path is required.\n")
 
@@ -523,7 +719,7 @@ def author_tech(author_email,repo_id):
 @click.argument('directory', required=False, type=GitRepo())
 def key_person(directory,top):
     """
-    Identify the key perople for a repo based on number of commits.
+    Identify the key people for a repo based on number of commits.
 
     Firstly, the top X committers of all time
 
@@ -545,41 +741,6 @@ def key_person(directory,top):
     table = KospexUtils.key_person_prettytable()
     headers = table.field_names
 
-    # active = kquery.commit_stats(repo_id=kgit.get_repo_id(),days=90)
-    # active_dict = {d["author"]: d for d in active}
-    # key = "commits"
-    # total_active_commits = sum(item[key] for item in active if key in item)
-
-    # authors = kquery.commit_stats(repo_id=kgit.get_repo_id())
-    # key = "commits"
-    # total_commits = sum(item[key] for item in authors if key in item)
-    # print(f"Total commits: {total_commits}")
-    # for a in authors:
-    #     a['active_commits'] = active_dict.get(a['author'],{}).get('commits',0)
-    #     a['% commits'] = f"{a['commits'] / total_commits * 100:.1f}%"
-    #     # TODO - We only want to do this if we're printing the table, modify if we're dumping to CSV
-    #     a['last_commit'] = KospexUtils.extract_db_date(a['last_commit'])
-    #     a['first_commit'] = KospexUtils.extract_db_date(a['first_commit'])
-
-    #     if total_active_commits > 0:
-    #         a['% active'] = f"{a['active_commits'] / total_active_commits * 100:.1f}%"
-    #     else:
-    #         a['% active'] = "0%"
-
-    # authors_dict = {d["author"]: d for d in authors}
-
-    # for a in authors[:top]:
-    #     #a['active_commits'] = active_dict.get(a['author'],{}).get('commits',0)
-    #     table.add_row(KospexUtils.get_values_by_keys(a, headers))
-
-    # top_authors_dict = {d["author"]: d for d in authors[:top]}
-
-    # for a in active[:top]:
-    #     a_dict = top_authors_dict.get(a['author'])
-    #     if not a_dict:
-    #         a_dict = authors_dict[a['author']]
-    #         table.add_row(KospexUtils.get_values_by_keys(a_dict, headers))
-    #
     authors = kquery.key_person(repo_id=kgit.get_repo_id(),top=top)
 
     for a in authors:
@@ -910,10 +1071,136 @@ def metadata(file_type,repo_id,sync):
     if table:
         print(f"Files: {len(table.rows)}")
 
-@cli.command("version")
-def version():
-    """Print the version of Kospex CLI."""
-    click.echo(f"Kospex CLI version {VERSION}")
+@cli.command("upgrade-db")
+@click.option('-apply', is_flag=True, default=False, help="Confirm and apply upgrades kospex DB schema.")
+def upgrade_db(apply):
+    """
+    Perform an upgrade and apply DB changes for new versions.
+    Run without options, it inspects the database and current kospex schema and detects changes
+    -apply will execute the alter table commands and update the db.
+    """
+    click.echo(f"Kospex CLI version {Kospex.VERSION}")
+    # WARN about backups first
+    print("\nWARNING: backup your database before performing ANY upgrade.\nJust in case ...\n")
+    # Check versions
+    # run the db diff
+    # apply db diff
+    # output status
+
+    data = kospex.kospex_query.get_kospex_db_version()
+    version = 0
+    if data is not None:
+        # try:
+        #     version = int(data)
+        # except (ValueError, TypeError):
+        #     # Set to 0, meaning we're in an unknown state, but we'll try an upgrade
+        #     invalid_version = True
+        version = data
+        #print(f"INVALID kospex db version '{data}'.\nSetting to 0 for need of an upgrade")
+        print(f"kospex db version: {version}")
+    else:
+        print("We don't have a kospex db version, either deleted, or an older database version")
+        print("Setting to 0 for need of an upgrade")
+
+    if version < KospexSchema.KOSPEX_DB_VERSION:
+        versions_behind = KospexSchema.KOSPEX_DB_VERSION - version
+        print(f"database is {versions_behind} versions behind.")
+
+    print()
+
+    alter_db_changes = []
+
+    rows = kospex.kospex_db.query("SELECT sql, tbl_name FROM sqlite_master where type = 'table'", [])
+
+    for r in rows:
+        tbl_name = r.get("tbl_name")
+        sql = r.get("sql")
+        print(f"tbl_name: {tbl_name}")
+        #print(KospexUtils.parse_sql_create_columns(sql))
+        if current_create := KospexSchema.DB_CREATE_STATEMENTS.get(tbl_name):
+            alter_commands = KospexSchema.generate_alter_table(sql,current_create,tbl_name)
+            if alter_commands:
+                print(f"Alter commands for table {tbl_name}")
+                print(alter_commands)
+                if alter_commands:
+                    alter_db_changes.extend(alter_commands)
+                    print("Adding to list of changes")
+            else:
+                print(f"No changes for table {tbl_name}")
+
+        else:
+            print("WARNING: table '{tbl_name}' is NOT a kospex databse table")
+
+        print()
+
+    print(f"Changes required: {len(alter_db_changes)}")
+    for change in alter_db_changes:
+        print(change)
+
+    if apply:
+        # Make the changes
+        print("Starting alter tables")
+        if len(alter_db_changes) > 0:
+            results = kospex.apply_alter_table_commands(alter_db_changes)
+            errors = 0
+            for item in results:
+                if item.get("error"):
+                    print(f"error: {item.get('error')} - message: {item.get('error')}")
+                    errors += 1
+
+            if errors > 0:
+               print("WARNING: Errors in applying the alter table")
+            else:
+                # update db version
+                rec = {
+                    "key": KospexSchema.KOSPEX_DB_VERSION_KEY,
+                    "value": str(KospexSchema.KOSPEX_DB_VERSION),
+                    "format": "INTEGER",
+                    "latest": 1
+                }
+                kospex.kospex_db.table(KospexSchema.TBL_KOSPEX_CONFIG).upsert(rec, pk=["key", "latest"])
+
+                print("Changes applied.")
+
+        else:
+                print("Nothing to apply.")
+
+
+    elif len(alter_db_changes) > 0:
+        print("\nThe above is the dry run of the changes, to apply them:")
+        print("> kospex upgrade-db -apply\n")
+        print("\nWARNING: BACKUP YOUR DATABASE FIRST!\n")
+    else:
+        print("No changes required, up to date.")
+
+@cli.command("advisory-history")
+@click.option('-ecosystem', type=click.STRING, help="E.g. npm, pypi")
+@click.option('-package', type=click.STRING, help="Name of package")
+def advisory_history(ecosystem, package):
+    """
+    Show the advisories for the previous X (default 10) versions of the package
+    """
+    if ecosystem and package:
+        print(f"Fetching advisories for {package} in {ecosystem}...")
+        # Fetch advisories logic here
+        data = kospex.dependencies.deps_dev_package(ecosystem, package)
+        kd = kospex.dependencies
+
+        import pprint
+        #pprint.PrettyPrinter(indent=4).pprint(data)
+        for p in data.get("versions"):
+            print(p.get("purl"))
+            parts = kd.extract_purl(p.get("purl"))
+            package_info = kd.deps_dev(parts["ecosystem"],
+                parts["package_name"],
+                parts["package_version"]
+            )
+            pprint.PrettyPrinter(indent=4).pprint(package_info)
+            print("\n")
+            #pprint.PrettyPrinter(indent=4).pprint(kd.extract_purl(p.get("purl")))
+    else:
+        print("Please provide both ecosystem and package names.")
+
 
 @cli.command("system-status")
 def status():
@@ -935,6 +1222,7 @@ def status():
 
     print("Database table version status\n")
 
+
     print("Installed tool status")
     print("---------------------")
     installed = which('scc')
