@@ -4,7 +4,8 @@ from sqlite_utils import Database
 import kospex_utils as KospexUtils
 
 # Definitions of the kospex DB tables
-
+TBL_BRANCHES = "branches"
+TBL_BRANCH_HISTORY = "branch_history"
 TBL_COMMITS = "commits"
 TBL_COMMIT_FILES = "commit_files"
 TBL_COMMIT_METADATA = "commit_metadata"
@@ -14,20 +15,32 @@ TBL_DEPENDENCY_DATA = "dependency_data"
 TBL_URL_CACHE = "url_cache"
 TBL_KRUNNER = "krunner"
 TBL_OBSERVATIONS = "observations"
-TBL_MAILMAP = "mailmaps"
 TBL_REPOS = "repos"
 # Experimental tables
 TBL_KOSPEX_META = "kospex_meta"
 TBL_GROUPS = "kospex_groups"
+TBL_KOSPEX_CONFIG = "kospex_config"
+# Not yet implemented
+TBL_MAILMAP = "mailmaps"
+
+
 
 KOSPEX_TABLES = [ TBL_COMMITS, TBL_COMMIT_FILES, TBL_COMMIT_METADATA, TBL_FILE_METADATA,
-                TBL_REPO_HOTSPOTS, TBL_DEPENDENCY_DATA, TBL_URL_CACHE,
-                TBL_KRUNNER, TBL_OBSERVATIONS, TBL_REPOS, TBL_KOSPEX_META, TBL_GROUPS ]
+                TBL_REPO_HOTSPOTS, TBL_DEPENDENCY_DATA, TBL_URL_CACHE, TBL_KRUNNER,
+                TBL_OBSERVATIONS, TBL_REPOS, TBL_KOSPEX_META, TBL_GROUPS, TBL_KOSPEX_CONFIG ]
 
-# The following is tables with a repo_id
+# The following are tables with a repo_id
 REPO_TABLES = [ TBL_COMMITS, TBL_COMMIT_FILES, TBL_COMMIT_METADATA, TBL_FILE_METADATA,
-                TBL_REPO_HOTSPOTS, TBL_DEPENDENCY_DATA, TBL_KRUNNER, TBL_OBSERVATIONS, TBL_REPOS, TBL_GROUPS ]
+                TBL_REPO_HOTSPOTS, TBL_DEPENDENCY_DATA, TBL_KRUNNER, TBL_OBSERVATIONS, TBL_REPOS,
+                TBL_GROUPS, TBL_BRANCHES, TBL_BRANCH_HISTORY ]
 
+# Mapping of table name to create statement is below the create statement definitions in:
+# DB_CREATE_STATEMENTS
+
+# KOSPEX_DB_VERSION will be updated every time we updated the schema
+KOSPEX_DB_VERSION=2
+KOSPEX_DB_VERSION_KEY = "KOSPEX_DB_VERSION_KEY"
+# Version 1, we're drawing a line in the sand as of 2025-02-16
 
 # Table data structure inspired by Mergestat sync 'git-commits'
 # https://github.com/mergestat/syncs/blob/main/syncs/git-commits/schema.sql
@@ -143,6 +156,37 @@ SQL_CREATE_COMMIT_METADATA = f'''CREATE TABLE IF NOT EXISTS [{TBL_COMMIT_METADAT
     [_repo_id] TEXT
     )'''
 
+# This table will store the current branch data for a repository
+SQL_CREATE_BRANCHES = f'''CREATE TABLE IF NOT EXISTS [{TBL_BRANCHES}] (
+    [count] INTEGER,
+    [branch_data] TEXT, -- JSON array of branch names
+    [hash] TEXT,        -- hash of the commit
+    [_repo_id] TEXT,
+    [_git_server] TEXT,
+    [_git_owner] TEXT,
+    [_git_repo] TEXT,
+    [created_at] DEFAULT CURRENT_TIMESTAMP,
+    [last_sync] TEXT,  -- date of last kospex sync
+    [last_seen] TEXT,  -- date of last commit, to be updated by the sync
+    PRIMARY KEY(_repo_id)
+    )'''
+
+# This table will store the HISTORIC branch data for a repository
+SQL_CREATE_BRANCH_HISTORY = f'''CREATE TABLE IF NOT EXISTS [{TBL_BRANCH_HISTORY}] (
+    [count] INTEGER,
+    [branch_data] TEXT, -- JSON array of branch names
+    [hash] TEXT,        -- hash of the commit
+    [_repo_id] TEXT,
+    [_git_server] TEXT,
+    [_git_owner] TEXT,
+    [_git_repo] TEXT,
+    [created_at] DEFAULT CURRENT_TIMESTAMP,
+    [last_sync] TEXT,  -- date of last kospex sync
+    [last_seen] TEXT,  -- date of last commit, to be updated by the sync
+    PRIMARY KEY(_repo_id, hash)
+    )'''
+
+
 # We're going to capture additional data about the dependencies in a file
 #SQL_PK_DEPENDENCY_DATA = '''PRIMARY KEY(_repo_id,hash,file_path,package_type,package_name,package_version)'''
 SQL_CREATE_DEPENDENCY_DATA = f'''CREATE TABLE IF NOT EXISTS [{TBL_DEPENDENCY_DATA}] (
@@ -206,6 +250,7 @@ SQL_CREATE_URL_CACHE = f'''CREATE TABLE IF NOT EXISTS [{TBL_URL_CACHE}] (
 #    )'''
 
 SQL_CREATE_OBSERVATIONS = f'''CREATE TABLE  IF NOT EXISTS [{TBL_OBSERVATIONS}] (
+    [uuid] TEXT,            -- unique identifier for the observation
     [hash] TEXT,             -- hash of the commit
     [file_path] TEXT,        -- file path in the repo (if applicable, can be repo level)
     [format] TEXT,           -- format type e.g. JSON, JSONL, CSV, LINE
@@ -222,10 +267,11 @@ SQL_CREATE_OBSERVATIONS = f'''CREATE TABLE  IF NOT EXISTS [{TBL_OBSERVATIONS}] (
     [_git_owner] TEXT,
     [_git_repo] TEXT,
     [_repo_id] TEXT,
-    PRIMARY KEY(_repo_id,hash,file_path,observation_key)
+    PRIMARY KEY(_repo_id,hash,file_path,observation_key,latest)
     )'''
 
 # TODO - This table has not been set up properly or created yet
+# Unsure if we still need this ... 2025-02-16
 SQL_CREATE_KOSPEX_META = f'''CREATE TABLE  IF NOT EXISTS [{TBL_KOSPEX_META}] (
     [format] TEXT,           -- format type e.g. JSON, JSONL, CSV, LINE
     [latest] INTEGER,        -- 1 if this is the latest version of the metadata, 0 otherwise
@@ -234,9 +280,21 @@ SQL_CREATE_KOSPEX_META = f'''CREATE TABLE  IF NOT EXISTS [{TBL_KOSPEX_META}] (
     [_git_owner] TEXT,
     [_git_repo] TEXT,
     [_repo_id] TEXT,
-    PRIMARY KEY(_repo_id,hash,file_path,observation_key)
+    PRIMARY KEY(_repo_id,hash,file_path)
     )'''
 
+#SQL_CREATE_KOSPEX_CONFIG = f'''CREATE TABLE IF NOT EXISTS [{TBL_KOSPEX_CONFIG}] (
+SQL_CREATE_KOSPEX_CONFIG = f'''CREATE TABLE IF NOT EXISTS [{TBL_KOSPEX_CONFIG}] (
+    [format] TEXT,           -- format type e.g. JSON, JSONL, CSV, TEXT, INTEGER, YAML
+    [key] TEXT,              -- config item key, similar to a key in a dict
+    [value] TEXT,            -- config item value, similar to a value in a dict, format specified in 'format'
+    [latest] INTEGER,        -- 1 if this is the latest version of the metadata, 0 otherwise
+    [created_at] DEFAULT CURRENT_TIMESTAMP,
+    [updated_at] DEFAULT CURRENT_TIMESTAMP,
+    PRIMARY KEY(key,latest)
+    )'''
+
+
 # Used for querying based on groups
 SQL_CREATE_GROUPS = f'''CREATE TABLE  IF NOT EXISTS [{TBL_GROUPS}] (
     [group_name] TEXT,  -- Name of the group
@@ -263,6 +321,24 @@ SQL_CREATE_MAILMAP = f'''CREATE TABLE  IF NOT EXISTS [{TBL_MAILMAP}] (
     PRIMARY KEY(_repo_id,email,committer_email)
     )'''
 
+DB_CREATE_STATEMENTS = {
+    TBL_COMMITS: SQL_CREATE_COMMITS,
+    TBL_COMMIT_FILES: SQL_CREATE_COMMIT_FILES,
+    TBL_COMMIT_METADATA: SQL_CREATE_COMMIT_METADATA,
+    TBL_FILE_METADATA: SQL_CREATE_FILE_METADATA,
+    TBL_REPO_HOTSPOTS: SQL_CREATE_REPO_HOTSPOTS,
+    TBL_DEPENDENCY_DATA: SQL_CREATE_DEPENDENCY_DATA,
+    TBL_URL_CACHE: SQL_CREATE_URL_CACHE,
+    TBL_KRUNNER: SQL_CREATE_KRUNNER,
+    TBL_OBSERVATIONS: SQL_CREATE_OBSERVATIONS,
+    TBL_REPOS: SQL_CREATE_REPOS,
+    TBL_KOSPEX_META: SQL_CREATE_KOSPEX_META,
+    TBL_GROUPS: SQL_CREATE_GROUPS,
+    TBL_KOSPEX_CONFIG: SQL_CREATE_KOSPEX_CONFIG,
+    TBL_BRANCHES: SQL_CREATE_BRANCHES,
+    TBL_BRANCH_HISTORY: SQL_CREATE_BRANCH_HISTORY,
+}
+
 # Functions for SQLite stuff
 
 def dict_factory(cursor, row):
@@ -295,9 +371,20 @@ def connect_or_create_kospex_db():
     kospex_db.execute(SQL_CREATE_OBSERVATIONS)
     kospex_db.execute(SQL_CREATE_REPOS)
     kospex_db.execute(SQL_CREATE_GROUPS)
+    kospex_db.execute(SQL_CREATE_KOSPEX_CONFIG)
+
+    kospex_db.execute(SQL_CREATE_BRANCHES)
+    kospex_db.execute(SQL_CREATE_BRANCH_HISTORY)
 
     # TODO - look at moving all table creates to "create if not exits"
 
+    if new_db:
+        # Set the database version
+        kospex_db.execute(
+            f"INSERT INTO {TBL_KOSPEX_CONFIG} (key, value, format, latest) VALUES (?, ?, ?, ?)",
+            [KOSPEX_DB_VERSION_KEY, str(KOSPEX_DB_VERSION), 'INTEGER', 1]
+        )
+
     return kospex_db
 
 def drop_table(table):
@@ -363,3 +450,153 @@ def metadata_rows_from_repo_files(files):
         #print(filtered_dict)
 
     return rows
+
+    # Claude.ai helped out with this one below
+    # Seems to work, and generates an alter table per column added
+    # TODO .. make sure all the old table columns are in the new table as sanity checking
+
+def generate_alter_table(old_create_sql, new_create_sql,tbl_name):
+
+    def extract_columns(create_sql):
+        start = create_sql.find('(') + 1
+        end = create_sql.rindex(')')
+        columns_section = create_sql[start:end]
+
+        # Split into lines and process each line
+        columns = []
+        for line in columns_section.split('\n'):
+            line = line.strip()
+            if not line or line.startswith('PRIMARY KEY'):
+                continue
+
+            # Remove any inline comments
+            if '--' in line:
+                line = line.split('--')[0].strip()
+
+            # Remove trailing comma if present
+            if line.endswith(','):
+                line = line[:-1].strip()
+
+            # Skip empty lines after processing
+            if not line:
+                continue
+
+            # Get column name from square brackets
+            if '[' in line:
+                col_name = line[line.find('[')+1:line.find(']')]
+                columns.append((col_name, line))
+
+        # TODO - Add as debug log
+        #print(f"Processed columns: {columns}")
+        return columns
+
+    old_cols = extract_columns(old_create_sql)
+    new_cols = extract_columns(new_create_sql)
+
+    old_names = [name for name, _ in old_cols]
+    new_names = [name for name, _ in new_cols]
+
+    # TODO - Add as debug log
+    #print(f"\nOld column names: {old_names}")
+    #print(f"New column names: {new_names}")
+
+    added_cols = [(name, def_) for name, def_ in new_cols if name not in old_names]
+
+    # TODO - Add as debug log
+    #print(f"Added columns: {added_cols}")
+
+    alter_commands = []
+    for _, col_def in added_cols:
+        alter_commands.append(f"ALTER TABLE [{tbl_name}] ADD COLUMN {col_def}\n")
+
+    return alter_commands
+
+def validate_square_brackets2(create_sql):
+    # Get the columns section
+    start = create_sql.find('(') + 1
+    end = create_sql.rindex(')')
+    columns_section = create_sql[start:end]
+
+    # Check each line
+    invalid_columns = []
+
+    for line in columns_section.split('\n'):
+        line = line.strip()
+        # Skip empty lines, comments, or PRIMARY KEY
+        if not line or line.startswith('--') or line.startswith('PRIMARY KEY'):
+            continue
+
+        # Remove comments
+        if '--' in line:
+            line = line.split('--')[0].strip()
+
+        # Split line by commas to handle multiple columns on one line
+        # but be careful not to split on commas inside parentheses (for types like VARCHAR(255))
+        columns_in_line = []
+        current_column = ""
+        paren_level = 0
+
+        for char in line:
+            if char == '(':
+                paren_level += 1
+                current_column += char
+            elif char == ')':
+                paren_level -= 1
+                current_column += char
+            elif char == ',' and paren_level == 0:
+                # End of a column definition
+                if current_column.strip():
+                    columns_in_line.append(current_column.strip())
+                current_column = ""
+            else:
+                current_column += char
+
+        # Add the last column if there is one
+        if current_column.strip():
+            columns_in_line.append(current_column.strip())
+
+        # Check each column declaration
+        for col_decl in columns_in_line:
+            # Check if the column declaration starts with a square bracket
+            if not col_decl.strip().startswith('['):
+                invalid_columns.append(col_decl.strip())
+
+    if invalid_columns:
+        print("The following column declarations are missing square brackets:")
+        for col in invalid_columns:
+            print(f"  {col}")
+        return False
+    return True
+
+def validate_square_brackets(create_sql):
+    # Get the columns section
+    start = create_sql.find('(') + 1
+    end = create_sql.rindex(')')
+    columns_section = create_sql[start:end]
+
+    # Check each line
+    invalid_lines = []
+    for line in columns_section.split('\n'):
+        line = line.strip()
+        # Skip empty lines, comments, or PRIMARY KEY
+        if not line or line.startswith('--') or line.startswith('PRIMARY KEY'):
+            continue
+        # Remove comments and trailing comma
+        if '--' in line:
+            line = line.split('--')[0].strip()
+        if line.endswith(','):
+            line = line[:-1].strip()
+        # Skip if empty after cleaning
+        if not line:
+            continue
+
+        # Check if the line starts with a square bracket
+        if not line.startswith('['):
+            invalid_lines.append(line)
+
+    if invalid_lines:
+        print("The following lines are missing square brackets:")
+        for line in invalid_lines:
+            print(f"  {line}")
+        return False
+    return True
